
int homePosPin = A3;
int sensorPinLeft = A0;   
int sensorPinRight = A1;
int ledPin = 13;

enum Position {
  LOCKEDEND,
  LOCKEDHOME_R,
  LOCKEDHOME_M,
  LOCKEDHOME_L,
  INLOCK,
  UNLOCKEDHOME_R,
  UNLOCKEDHOME_M,
  UNLOCKEDHOME_L,
  UNLOCKEND,
  UNDEFINED,
};

enum HomePosition {
  HOME_L,
  HOME_M,
  HOME_R,
  NOHOME,
};


String positionToString(const int pos) {
  switch (pos) {
    case LOCKEDEND : return String("LOCKEDEND"); break;
    case LOCKEDHOME_R : return String("LOCKED_HOME_RIGHT"); break;
    case LOCKEDHOME_M : return String("LOCKED_HOME_MIDDLE"); break;
    case LOCKEDHOME_L : return String("LOCKED_HOME_LEFT"); break;
    case INLOCK : return String("INLOCK"); break;
    case UNLOCKEDHOME_R : return String("UNLOCKED_HOME_RIGHT"); break;
    case UNLOCKEDHOME_M : return String("UNLOCKED_HOME_MIDDLE"); break;
    case UNLOCKEDHOME_L : return String("UNLOCKED_HOME_LEFT"); break;
    case UNLOCKEND : return String("UNLOCK_END"); break;
    case UNDEFINED : return String("UNDEFINED"); break;
  }
}


Position currentPosition = UNDEFINED;
void updatePosition() {
  sensHomePosition();
  int hp = sensHomePosition();
  

  /***** STATE MACHINE ******/

  if (currentPosition == LOCKEDEND) {
    if (hp == HOME_R) {
      currentPosition = LOCKEDHOME_R;
      return;
    }
    if (hp == NOHOME) {
      // same position
      return;
    }
    currentPosition = UNDEFINED;
  }
  
  if (currentPosition == LOCKEDHOME_R) {
    if (hp == NOHOME) {
      currentPosition = LOCKEDEND;
      return;
    }
    if (hp == HOME_M) {
      currentPosition = LOCKEDHOME_M;
     return; 
    }
    if (hp == HOME_R) {
      // same position
      return;
    }
    currentPosition = UNDEFINED;
  }

  if (currentPosition == LOCKEDHOME_M) {
    if (hp == HOME_L) {
      currentPosition = LOCKEDHOME_L;
      return;
    }
    if (hp == HOME_R) {
      currentPosition = LOCKEDHOME_R;
      return;
    }
    if (hp == HOME_M) {
      // same position
      return;
    }
    currentPosition = UNDEFINED;
  }

  if (currentPosition == LOCKEDHOME_L) {
    if (hp == HOME_M) {
      currentPosition = LOCKEDHOME_M;
      return;
    }
    if (hp == NOHOME) {
      currentPosition = INLOCK;
      return;
    }
    if (hp == HOME_L) {
      // same position
      return;
    }
    currentPosition = UNDEFINED;
  }

  if (currentPosition == INLOCK) {
    if (hp == HOME_L) {
      currentPosition = LOCKEDHOME_L;
      return;
    }
    if (hp == HOME_R) {
      currentPosition = UNLOCKEDHOME_R;
      return;
    }
    if (hp == NOHOME) {
      // same position
      return;
    }
    currentPosition = UNDEFINED;
  }

  if (currentPosition == UNLOCKEDHOME_R) {
    if (hp == HOME_M) {
      currentPosition = UNLOCKEDHOME_M;
      return;
    }
    if (hp == NOHOME) {
      currentPosition = INLOCK;
      return;
    }
    if (hp == HOME_R) {
      // same position
      return;
    }
    currentPosition = UNDEFINED;
  }

  if (currentPosition == UNLOCKEDHOME_M) {
    if (hp == HOME_R) {
      currentPosition = UNLOCKEDHOME_R;
      return;
    }
    if (hp == HOME_L) {
      currentPosition = UNLOCKEDHOME_L;
      return;
    }
    if (hp == HOME_M) {
      // same position
      return;
    }
    currentPosition = UNDEFINED;
  }

  if (currentPosition == UNLOCKEDHOME_L) {
    if (hp == HOME_M) {
      currentPosition = UNLOCKEDHOME_M;
      return;
    }
    if (hp == NOHOME) {
      currentPosition = UNLOCKEND;
      return;
    }
    if (hp == HOME_L) {
      // same position
      return;
    }
    currentPosition = UNDEFINED;
  }

  if (currentPosition == UNLOCKEND) {
    if (hp == HOME_L) {
      currentPosition = UNLOCKEDHOME_L;
      return;
    }
    if (hp == NOHOME) {
      //same position
      return;
    }
    currentPosition = UNDEFINED;
  }

        
}


int sensHomePosition() {
  /* calibration values 
   * analog voltage using a 2k resistor. 
   */
  const int nohome_low = 120;
  const int nohome_high = 125;
  const int left_low = 160;
  const int left_high = 164;
  const int middle_low = 470;
  const int middle_high = 472;
  const int right_low = 245;
  const int right_high = 248;

  /**********************/
  /* sample analog value */
  /*
  int nsamples = 100;
  unsigned long ssum = 0;
  for (int i=0; i<nsamples; i++) {
    int av = analogRead(homePosPin);
    ssum += av;
  }
  int avalue = ssum / nsamples;
  */


  /* a position has to be sensed several times in a row
   * before it gets recognized so */
  int lpos = -1;
  int counted = 0;
  for (;;) {
    int av = analogRead(homePosPin);
    int cpos = -1;
    if (av >= nohome_low && av <= nohome_high) {
      cpos = NOHOME;
    }
    if (av >= left_low && av <= left_high) {
      cpos = HOME_L;
    }
    if (av >= middle_low && av <= middle_high) {
      cpos = HOME_M;
    }
    if (av >= right_low && av <= right_high) {
      cpos = HOME_R;
    }

    if ( cpos != -1 && cpos == lpos) {
      counted++;
    } else {
      counted = 0;
      lpos = cpos;
    }

    if (counted == 5) {
      return cpos;
    }
  }
  
} 
  
  
void setup() {
  Serial.begin(9600);
  // declare the ledPin as an OUTPUT:
  pinMode(ledPin, OUTPUT);  
  currentPosition = LOCKEDEND; 
}

void loop() {
  /* read analog motor input */
  int leftTurnV = analogRead(sensorPinLeft);    
  int rightTurnV = analogRead(sensorPinRight);
  
  /* print raw values to serial 
  Serial.print("left: ");
  Serial.print(leftTurnV);
  Serial.print(" right: ");
  Serial.print(rightTurnV);
  Serial.println();
  */
  updatePosition();
  String cPosS = positionToString(currentPosition);
  Serial.println(cPosS);
  
  /* wait 100 milliseconds */         
}

