/*
Firmware for /dev/lock
 */
 
 
/****** Sensor and LED Pins ******/
#define PDOORLED 13
#define POPENSENSOR 2
#define HOMEPOSPIN 3
/***************************/


/***** Motor Interface *****/
#define MPWM 3         // Jeenode Port1-4 I(rq) Arduino 3
#define MDIR 4         // Jeenode Port1 D Arduino 4
#define MSEN A0        // Jeenode Port1 A Arduino A0
#define UDELAY 2       // RampUp delay time 
#define DDELAY 2       // RampDown delay time 
#define RDELAY 100     // Fullspeed Run time 
#define BREAKDELAY 150 // ms which is required for breaking from full speed
#define SENSCAL 1.62   // Current Sensor Calibration Factor 1.62 (bc -l)
/***************************/


/****** PROTOCOL *********/
// commands, Strings are used for comparrison
String DOLOCK = "DOLOCK";
String DOUNLOCK = "DOUNLOCK";
String GETSTATE = "GETSTATE";
#define STATE "STATE" // response to GETSTATE
String INFO = "INFO";
// events
#define WASOPENED "WASOPENED"
#define WASCLOSED "WASCLOSED"
#define WASLOCKED "WASLOCKED"
#define WASUNLOCKED "WASUNLOCKED"
/*************************/


/***** POSITION STATE MACHINE ******/

enum Position_R {
  LOCKEDEND,
  LOCKEDHOME_R,
  LOCKEDHOME_M,
  LOCKEDHOME_L,
  INLOCK,
  UNLOCKEDHOME_R,
  UNLOCKEDHOME_M,
  UNLOCKEDHOME_L,
  UNLOCKEND,
  UNDEFINED,
};
// the current logical position of the lock
int currentPosition = UNDEFINED;

/* returns the direction to turn in to reach the [to] position */
int getDirectionForPos(int from, int to) {
  if (from < to) {
    return -1;
  } else if (from > to) {
   return 1;
  } else {
    return 0; 
  }
}

enum HomePosition {
  HOME_L,
  HOME_M,
  HOME_R,
  NOHOME,
  HOMEERROR
};


String positionToString(const int pos) {
  switch (pos) {
    case LOCKEDEND : return String("LOCKEDEND"); break;
    case LOCKEDHOME_R : return String("LOCKED_HOME_RIGHT"); break;
    case LOCKEDHOME_M : return String("LOCKED_HOME_MIDDLE"); break;
    case LOCKEDHOME_L : return String("LOCKED_HOME_LEFT"); break;
    case INLOCK : return String("INLOCK"); break;
    case UNLOCKEDHOME_R : return String("UNLOCKED_HOME_RIGHT"); break;
    case UNLOCKEDHOME_M : return String("UNLOCKED_HOME_MIDDLE"); break;
    case UNLOCKEDHOME_L : return String("UNLOCKED_HOME_LEFT"); break;
    case UNLOCKEND : return String("UNLOCK_END"); break;
    case UNDEFINED : return String("UNDEFINED"); break;
  }
}

/**************************************/


/* Updates the global variable currentPosition.
 * Implementation of a state machine */
void updatePosition() {
  int hp = sensHomePosition();

  /***** STATES ******/

  if (currentPosition == LOCKEDEND) {
    if (hp == HOME_R) {
      currentPosition = LOCKEDHOME_R;
      return;
    }
    if (hp == NOHOME) {
      // same position
      return;
    }
    currentPosition = UNDEFINED;
  }
  
  if (currentPosition == LOCKEDHOME_R) {
    if (hp == NOHOME) {
      currentPosition = LOCKEDEND;
      return;
    }
    if (hp == HOME_M) {
      currentPosition = LOCKEDHOME_M;
     return; 
    }
    if (hp == HOME_R) {
      // same position
      return;
    }
    currentPosition = UNDEFINED;
  }

  if (currentPosition == LOCKEDHOME_M) {
    if (hp == HOME_L) {
      currentPosition = LOCKEDHOME_L;
      return;
    }
    if (hp == HOME_R) {
      currentPosition = LOCKEDHOME_R;
      return;
    }
    if (hp == HOME_M) {
      // same position
      return;
    }
    currentPosition = UNDEFINED;
  }

  if (currentPosition == LOCKEDHOME_L) {
    if (hp == HOME_M) {
      currentPosition = LOCKEDHOME_M;
      return;
    }
    if (hp == NOHOME) {
      currentPosition = INLOCK;
      return;
    }
    if (hp == HOME_L) {
      // same position
      return;
    }
    currentPosition = UNDEFINED;
  }

  if (currentPosition == INLOCK) {
    if (hp == HOME_L) {
      currentPosition = LOCKEDHOME_L;
      return;
    }
    if (hp == HOME_R) {
      currentPosition = UNLOCKEDHOME_R;
      return;
    }
    if (hp == NOHOME) {
      // same position
      return;
    }
    currentPosition = UNDEFINED;
  }

  if (currentPosition == UNLOCKEDHOME_R) {
    if (hp == HOME_M) {
      currentPosition = UNLOCKEDHOME_M;
      return;
    }
    if (hp == NOHOME) {
      currentPosition = INLOCK;
      return;
    }
    if (hp == HOME_R) {
      // same position
      return;
    }
    currentPosition = UNDEFINED;
  }

  if (currentPosition == UNLOCKEDHOME_M) {
    if (hp == HOME_R) {
      currentPosition = UNLOCKEDHOME_R;
      return;
    }
    if (hp == HOME_L) {
      currentPosition = UNLOCKEDHOME_L;
      return;
    }
    if (hp == HOME_M) {
      // same position
      return;
    }
    currentPosition = UNDEFINED;
  }

  if (currentPosition == UNLOCKEDHOME_L) {
    if (hp == HOME_M) {
      currentPosition = UNLOCKEDHOME_M;
      return;
    }
    if (hp == NOHOME) {
      currentPosition = UNLOCKEND;
      return;
    }
    if (hp == HOME_L) {
      // same position
      return;
    }
    currentPosition = UNDEFINED;
  }

  if (currentPosition == UNLOCKEND) {
    if (hp == HOME_L) {
      currentPosition = UNLOCKEDHOME_L;
      return;
    }
    if (hp == NOHOME) {
      //same position
      return;
    }
    currentPosition = UNDEFINED;
  }
  
  if (currentPosition == UNDEFINED) {
    initPosition();
  }
        
}


int sensHomePosition() {
  /* calibration values 
   * analog voltage using a 2k resistor. 
   */
  const int nohome_low = 894;
  const int nohome_high = 902;
  const int left_low = 856;
  const int left_high = 864;
  const int middle_low = 545;
  const int middle_high = 555;
  const int right_low = 769;
  const int right_high = 778;

  // the position must be measured #numCounts in a row to be valid
  // this removes possible noise
  const int numCounts = 5;

  // the absolut number of samples before measurement times out
  const int maxSamples = 300;

  /* a position has to be sensed several times in a row
   * before it gets recognized so */
  int lpos = -1;
  int counted = 0;
  for (int i=0;;i++) {
    int av = analogRead(HOMEPOSPIN);
    int cpos = -1;
    if (av >= nohome_low && av <= nohome_high) {
      cpos = NOHOME;
    }
    if (av >= left_low && av <= left_high) {
      cpos = HOME_L;
    }
    if (av >= middle_low && av <= middle_high) {
      cpos = HOME_M;
    }
    if (av >= right_low && av <= right_high) {
      cpos = HOME_R;
    }

    if ( cpos != -1 && cpos == lpos) {
      counted++;
    } else {
      counted = 0;
      lpos = cpos;
    }

    if (counted == 5) {
      return cpos;
    }

    /* can't measure the home position.
       too much noise or not connected */
    if (i > maxSamples) {
      return HOMEERROR;
    }
  }
  
}


void startMotor(int direction) {
  // set the direction on the H-Bridge
  if (direction == 1) {
    digitalWrite(MDIR, LOW);
  } else if (direction == -1) {
    digitalWrite(MDIR, HIGH);
  }

  // start motor  
  analogWrite(MPWM, 170);
}


void stopMotor() {

  // stop motor
  analogWrite(MPWM, 0);
  delay(BREAKDELAY);
  
}

void driveToPosition(int pos) {
  // the direction to drive the motor in [-1,0,1]
  int direction = getDirectionForPos(currentPosition, pos);

  Serial.print("drive to direction: ");
  Serial.println(direction);

  // no direction to drive to
  if (direction == 0 || currentPosition == UNDEFINED) {
    return;
  }

  startMotor(direction);

  while (currentPosition != pos && currentPosition != UNDEFINED) {
    updatePosition();
  }
  
  stopMotor();

  updatePosition();
}

  
/* locks the door, drive motor */
void lockDoor() {
  // don't close when the door isn't shut properly
  if (isOpen()) return;
  driveToPosition(LOCKEDHOME_M);
}


/* unlock the door, drive motor */
void unlockDoor() {
  driveToPosition(UNLOCKEDHOME_M);
}


/* the should not reside in the INLOCK position which
 * 
void fixInLock() {
  
}


/* return the locked state */
int isLocked() {
  updatePosition();
  switch (currentPosition) {
    case LOCKEDEND:;
    case LOCKEDHOME_R:;
    case LOCKEDHOME_M:;
    case LOCKEDHOME_L: return 1;
    default: return 0;
  }
  return 0;
}


/* return door open state, sensor */
int isOpen() {
  int openpin = digitalRead(POPENSENSOR);
  /* LOW means the switch is on, door closed */
  if (openpin == 0) {
    return 1;
  } else {
    return 0;
  }

}


/* The lock must not reside in the state INLOCK
 * since the lock status (locked/unlocked) is 
 * undefined in this position.
 * INLOCK may happen when thee door is locked
 * manually. If the lock is >10 seconds at this
 * position it will lock automaticall and move to
 * LOCKEDHOME_M.
 */
void fixInLock() {
  const int waitTime = 10 * 1000; // wait 10 seconds
  static unsigned long firstSenseTime = millis();
  static int isInINLOCK = false; 

  unsigned long ctime = millis();

  if (currentPosition != INLOCK) {
    isInINLOCK = false;
    return;
  } else {
    if (! isInINLOCK) {
      isInINLOCK = true;
      firstSenseTime = ctime;
    }
  }

  /* protect overflow */
  if (ctime < firstSenseTime) {
    firstSenseTime = ctime;
  }

  /* if the lock is > waitTime in INLOCK, l
    * lock the door for real. */
  if (ctime > (firstSenseTime + waitTime)) {
    lockDoor();    
  }
}


/* send data on serial about door and sensors */
void sendStateData() {
  Serial.print(STATE);
  Serial.print(": ");
  
  if (isLocked()) {
    Serial.print("LOCKED, ");
  } else {
    Serial.print("UNLOCKED, ");
  }
  
  if (isOpen()) {
    Serial.print("OPEN, ");
  } else {
    Serial.print("CLOSED, ");
  }
  
  Serial.println("");
}


/* send device and firmware info */
void sendInfo() {
  Serial.println("/dev/lock firmware v0.1");
}


/* handles commands from serial input */
void handleCommands() {
  if (Serial.available()) {
    String cbuffer;
    int cbuffer_len = 100;
    
    /* Get the command string */
    char c;
    for(int i=0; i<cbuffer_len; i++) {
      c = Serial.read();
      //FIXME \n is not detected
      if (c == ';' || c == '\n') {
        break;
      }
      cbuffer += c;
    }
    
    /*** Handle commands ****/
    if (cbuffer.equals(DOLOCK)) {
      lockDoor();
    }
    
    if (cbuffer.equals(DOUNLOCK)) {
      unlockDoor();
    }
    
    if (cbuffer.equals(GETSTATE)) {
      sendStateData();
    }
    
    if (cbuffer.equals(INFO)) {
      sendInfo();
    }
  }
}


/* detect and send events on serial */
void detectAndSendEvents() {
  static int oldIsLocked = isLocked();
  static int oldIsOpen = isOpen();
  
  // Wait this time after an event is fired
  // before sending a new one.
  const int eventDelay = 200;
  
  int cIsLocked = isLocked();
  int cIsOpen = isOpen();
  
  /*** the ordering of events is important ***/
  
  /* unlocked event */
  if ((cIsLocked != oldIsLocked) && (! cIsLocked)) {
     Serial.println(WASUNLOCKED);
  }

  /* door open/close event */
  if (cIsOpen != oldIsOpen) {
    if (cIsOpen) {
      Serial.println(WASOPENED);
    } else {
      Serial.println(WASCLOSED);
    }
    delay(eventDelay);
  }

  /* locked event */
  if ((cIsLocked != oldIsLocked) && (cIsLocked)) {
     Serial.println(WASLOCKED);
  }
  
  /* update states for next call */
  oldIsLocked = cIsLocked;
  oldIsOpen = cIsOpen;
}


void initDoor() {
  currentPosition = UNDEFINED; 

  if (!isOpen()) {
    //lockDoor();
  }
}


void initPosition() {
  Serial.println("init position");
  if (!isOpen()) {
    Serial.println("init drive motor");
    /* make sure the door is really shut */
    for (int i=0; i<10; i++) {
      delay(100);
      if (isOpen()) return;
    }
    
    /* drive the motor for 2.5 seconds */
    startMotor(1);
    delay(2500);
    stopMotor();
    
    currentPosition = LOCKEDEND;
  }  
}


void setup() {
  // initialize serial communication at 9600 bits per second:
  Serial.begin(9600);

  // make the pushbutton's pin an input:
  pinMode(POPENSENSOR, INPUT);
  pinMode(PDOORLED, OUTPUT);
 
  /* set up motor interface */
  pinMode(MPWM, OUTPUT);
  pinMode(MDIR, OUTPUT);
  digitalWrite(MDIR, LOW);
  digitalWrite(MPWM, LOW);

  Serial.println("init door");
  initDoor();
}


void loop() {
  /*
  Serial.print(positionToString(currentPosition));
  Serial.print(", ");
  Serial.print(sensHomePosition());
  Serial.print(", ");
  Serial.print(isOpen());
  Serial.println();
  */

  /* update the logical position */
  updatePosition();
  
  /* fix the invalid INLOCK position */
  fixInLock();

  /* Handle commands from the serial */
  handleCommands();
  
  /* Sense and send events */
  detectAndSendEvents();  
  
}

