###
# Copyright 2014, Aurel Wildfellner.
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software Foundation,
# Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
#

import os
import csv, hashlib, base64

from abc import ABCMeta, abstractmethod


class AccountDB(object):

    def __init__(self):
        self.accounts = []
        pass

    def loadDB(self, path):
        self.accounts = []
        with open(path, 'r') as csvfile:
            reader = csv.reader(csvfile)
            for row in reader:
                self.accounts.append(row)


    def saveDB(self, path):
        with open(path, 'w') as f:
            writer = csv.writer(f)
            writer.writerows(self.accounts)


    def identify(self, username, password):
        for account in self.accounts:
            if len(account) != 3:
                continue
            if account[0] == username:
                saltedpw = account[1] + password
                sha512h = hashlib.sha512()
                sha512h.update(bytes(saltedpw, "utf-8"))
                pwhash = base64.b64encode(sha512h.digest()).decode('utf-8')
                return (pwhash == account[2])
        return False


    def addAccount(self, username, password):
        salt = base64.b64encode(os.urandom(10)).decode('utf-8')
        saltedpw = salt + password
         
        sha512h = hashlib.sha512()
        sha512h.update(bytes(saltedpw, "utf-8"))
        pwhash = base64.b64encode(sha512h.digest()).decode('utf-8')

        account = [username, salt, pwhash]
        self.accounts.append(account)

