##
# Copyright 2014, Aurel Wildfellner.
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software Foundation,
# Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
#

from threading import Thread
import time

import zmq

from corefsm import EventQueue, LockSystemEvent, EventType, EventSource


class CmdAPIProto:

    port = 5574
    #cmds
    DOLOCK = "DOLOCK"
    DOUNLOCK = "DOUNLOCK"
    DOLOCKAFTERME = "DOLOCKAFTERME"
    INFO = "INFO"
    #return
    OK = "OK"
    ERROR = "ERROR"
    NOT_IMPLEMENTED = "NOT_IMPLEMENTED"


class CmdAPISever(object):

    port = 5574

    def __init__(self, eventQueue, door):
        self.eventQueue = eventQueue
        self.door = door
        self._context = zmq.Context()
        self._stopThread = False
        self._running = False


    def run(self):
        socket = self._context.socket(zmq.REP)
        socket.bind("tcp://127.0.0.1:%s" % CmdAPIProto.port)

        while not self._stopThread:
            while True:
                try:
                    msg = socket.recv_unicode(flags=zmq.NOBLOCK)
                    break
                except :
                    time.sleep(0.05)
                    pass
                if self._stopThread:
                    socket.close()
                    return

            if msg == CmdAPIProto.DOLOCK:
                evt = LockSystemEvent(EventSource.WEBINTERFACE, EventType.DOLOCK)
                self.eventQueue.enqueueEvent(evt)
                socket.send_unicode(CmdAPIProto.OK)

            elif msg == CmdAPIProto.DOUNLOCK:
                evt = LockSystemEvent(EventSource.WEBINTERFACE, EventType.DOUNLOCK)
                self.eventQueue.enqueueEvent(evt)
                socket.send_unicode(CmdAPIProto.OK)

            elif msg == CmdAPIProto.DOLOCKAFTERME:
                evt = LockSystemEvent(EventSource.WEBINTERFACE, 
                        EventType.DOLOCKAFTERME)
                self.eventQueue.enqueueEvent(evt)
                socket.send_unicode(CmdAPIProto.OK)

            elif msg == CmdAPIProto.INFO:
                infomsg = ""
                if self.door.isOpen():
                    infomsg += "open, "
                else:
                    infomsg += "closed, "
                if self.door.isLocked():
                    infomsg += "locked, "
                else:
                    infomsg += "unlocked, "
                socket.send_unicode(infomsg)

            else:
                socket.send_unicode(CmdAPIProto.NOT_IMPLEMENTED)


        socket.close()


    def start(self):
        if self._running:
            return

        self._thread = Thread(target=self.run)
        self._thread.start()
        self._running = True


    def stop(self):
        if self._running:
            self._stopThread = True
            self._thread.join()




class CmdAPIClient(object):

    def __init__(self):
        self._context = zmq.Context()
        self._socket = self._context.socket(zmq.REQ)
        self._connected = False
        self._addr = "tcp://127.0.0.1:%s" % CmdAPIProto.port

    @property
    def connected(self):
        return self._connected


    def connect(self):
        self._socket.connect(self._addr)
        self._connected = True

    def disconnect(self):
        self._socket.disconnect(self._addr)
        self._connected = False


    def sendCMD(self, cmd):
        if not self._connected:
            return "NO_CONNECTION"
        self._socket.send_unicode(cmd)
        rec = self._socket.recv_unicode()
        return rec


    def sendDOLOCK(self):
        return self.sendCMD(CmdAPIProto.DOLOCK)

    def sendDOUNLOCK(self):
        return self.sendCMD(CmdAPIProto.DOUNLOCK)

    def sendDOLOCKAFTERME(self):
        return self.sendCMD(CmdAPIProto.DOLOCKAFTERME)

    def sendINFO(self):
        return self.sendCMD(CmdAPIProto.INFO)

