#!/usr/bin/python3

##
# Copyright 2014, Aurel Wildfellner.
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software Foundation,
# Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
#

import time

from corefsm import *
from door import Door, MockupDoor, ArduinoDoor

from cmdapi import CmdAPISever


def eventMenuMockup(eventQueue, door):
    print("---------------------------------")
    print("(l)ock  (u)nlock  (a)LockAfterMe  (o)open  (c)close  (x)lockMan  (y)unlockMan  (i)nfo  (q)uit")
    print("># ", end="")

    inp = input()
    if len(inp) == 0:
        return True

    if inp[0] == 'l':
        ev = LockSystemEvent(EventSource.SHELL, EventType.DOLOCK)
        eventQueue.enqueueEvent(ev)
    elif inp[0] == 'u':
        ev = LockSystemEvent(EventSource.SHELL, EventType.DOUNLOCK)
        eventQueue.enqueueEvent(ev)
    elif inp[0] == 'a':
        ev = LockSystemEvent(EventSource.SHELL, EventType.DOLOCKAFTERME)
        eventQueue.enqueueEvent(ev)
    elif inp[0] == 'o':
        door.setOpenState(True)
    elif inp[0] == 'c':
        door.setOpenState(False)
    elif inp[0] == 'x':
        door.setLockedState(True)
    elif inp[0] == 'y':
        door.setLockedState(False)
    elif inp[0] == 'i':
        print("Door locked [" + str(door.isLocked()) + "]; Door open [" 
            + str(door.isOpen()) + "]")
    elif inp[0] == 'q':
        return False

    return True


def tryToConnectArduino(door):
    devices =  ["/dev/ttyACM" + str(x) for x in range(10)]
    devices += ["/dev/ttyUSB" + str(x) for x in range(10)]

    for device in devices:
        door.connectArduino(device)
        if door.isConnected:
            break


def main():

    eventQueue = EventQueue()
    #door = MockupDoor(eventQueue)

    door = ArduinoDoor(eventQueue)
    tryToConnectArduino(door)
    print("Arduino Connected: ", door.isConnected)

    fsm = LockSystemFSM(door, eventQueue)
    fsm.start()

    cmdAPIServer = CmdAPISever(eventQueue, door)
    cmdAPIServer.start()

    time.sleep(0.5)
    #while eventMenu(eventQueue, door):
    #    time.sleep(0.2)
    while True:
        time.sleep(1)
        if not door.isConnected:
            print("Arduino is disconnected! Reconnecting.... ", end="")
            tryToConnectArduino(door)
            if (door.isConnected):
                print("done!")
            else:
                print("failed!")

    cmdAPIServer.stop()
    fsm.stop()


if __name__ == "__main__":
    main()

