#!/usr/bin/python3

###
# Copyright 2014, Aurel Wildfellner.
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software Foundation,
# Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
#

from flask import Flask, render_template, request, url_for, redirect
from flask import Response, make_response
from flask_login import LoginManager, login_required, current_user, login_user

from user import User

import time, os

from cmdapi import CmdAPIClient, CmdAPIProto
from accountdb import AccountDB
import spaceapi

gDebugcounter = 0
devLockWebPath = "/var/www/devlockweb/"

cmdAPIClient = None
lastCMDReturnValue = (False, "")


#### Run from WSGI deamon #####
if True:
    ### the main Flask Application
    app = Flask(__name__)
    # setup key and crypto
    app.secret_key = os.urandom(2048)


    ### load account database
    accountDB = AccountDB()
    accountDB.loadDB(devLockWebPath + "accounts.csv")

    ### setup authentication
    loginManager = LoginManager()
    loginManager.init_app(app)
    @loginManager.user_loader
    def load_user(userid):
        if userid in User.allUsers:
            return User.allUsers[userid]
        else:
            return None
    # Redirect to login if any view is accessed 
    # without authentication.
    loginManager.login_view = "/login"
    # protect session cookie
    loginManager.session_protection = "strong"




def apiContext():
    global cmdAPIClient
    """Workaround to connect the api the wsgi context.
       This is needed to create zmqs context correctly."""
    if cmdAPIClient == None:
        cmdAPIClient = CmdAPIClient()
        cmdAPIClient.connect()


@app.route('/')
def mainForm():
    apiContext()
    return redirect(url_for('loginPage'))


@app.route('/maininterface', methods=['GET'])
@login_required
def mainInterface():
    global cmdAPIClient, gDebugcounter, lastCMDReturnValue
    apiContext()
    gDebugcounter += 1

    # string to be parsed for door information (locked/opened?)
    doorinfo = ""

    ### variables for the HTML template
    open_status = ""
    locked_status = ""
    ret_message = ""


    # print command return info if there is one
    if lastCMDReturnValue[0]:
        ret_message = lastCMDReturnValue[1]
        lastCMDReturnValue = (False, "")


    ### parse door info
    doorinfo = cmdAPIClient.sendINFO()
    infotokens = doorinfo.split(',')
    infotokens = [str.strip(t) for t in infotokens]
    if "locked" in infotokens:
        locked_status = "locked"
    elif "unlocked" in infotokens:
        locked_status = "unlocked"
    else:
        locked_status = "unknown"

    if "open" in infotokens:
        open_status = "open"
    elif "closed" in infotokens:
        open_status = "closed"
    else:
        open_status = "unknown"

    # compile page
    page = render_template('interface.html', 
            debug_counter=str(gDebugcounter),
            open_status=open_status,
            locked_status=locked_status,
            ret_message=ret_message)

    return page


@app.route('/login', methods=['GET'])
def loginPage():
    global cmdAPIClient, gDebugcounter, lastCMDReturnValue
    apiContext()

    # redirect a logged in user
    if current_user.is_authenticated():
        return redirect(url_for('mainInterface'))

    page = render_template('login.html')

    return page


@app.route('/login', methods=['POST'])
def loginPost():
    global cmdAPIClient, gDebugcounter, lastCMDReturnValue, accountDB
    apiContext()

    username = request.form["username"]
    password = request.form["password"]
    user = User(username, accountDB)
    isAuthed = user.login(password)
    if (isAuthed):
        login_user(user)
        return redirect(url_for('mainInterface'))
    else:
        return redirect(url_for('loginPage'))


@app.route('/cmd/lock')
@login_required
def cmdLock():
    global cmdAPIClient, lastCMDReturnValue
    apiContext()
    ret = cmdAPIClient.sendDOLOCK()
    lastCMDReturnValue = (True, "lock - " + ret)
    time.sleep(2)
    return redirect(url_for('mainInterface'))

@app.route('/cmd/unlock')
@login_required
def cmdUnlock():
    global cmdAPIClient, lastCMDReturnValue
    apiContext()
    ret = cmdAPIClient.sendDOUNLOCK()
    lastCMDReturnValue = (True, "unlock - " + ret)
    time.sleep(2)
    return redirect(url_for('mainInterface'))

@app.route('/cmd/lockafterme')
@login_required
def cmdLockAfterMe():
    global cmdAPIClient, lastCMDReturnValue
    apiContext()
    ret = cmdAPIClient.sendDOLOCKAFTERME()
    lastCMDReturnValue = (True, "lock after me - " + ret)
    time.sleep(0.5)
    return redirect(url_for('mainInterface'))

@app.route('/cmd/status/open')
@login_required
def cmdStatusOpen():
    spaceapi.setSpaceStatus(True)
    return redirect(url_for('mainInterface'))

@app.route('/cmd/status/close')
@login_required
def cmdStatusClose():
    spaceapi.setSpaceStatus(False)
    return redirect(url_for('mainInterface'))


def runWebInterface():
    """Standalone Run, NOT from WSGI deamon."""
    global app

    #WARNING, DEBUGGING ENABLES REMOTE CODE EXECUTION!!!!
    #FIXME
    app.debug = False
    #TODO
    #WARNING, DEBUGGING ENABLES REMOTE CODE EXECUTION!!!!
    app.run()
    #app.run(host='0.0.0.0')


def main():
    runWebInterface()


if __name__ == "__main__":
    main()

